@@ -14,12 +14,9 @@ use slumber_core::{
     database::{CollectionDatabase, Database},
     http::{
         BuildOptions, Exchange, HttpEngine, RequestRecord, RequestSeed,
-        ResponseRecord,
-    },
-    template::{
-        HttpProvider, Prompt, Prompter, Select, TemplateContext,
-        TriggeredRequestError,
+        ResponseRecord, TriggeredRequestError,
     },
+    render::{HttpProvider, Prompt, Prompter, Select, TemplateContext},
     util::MaybeStr,
 };
 use slumber_util::ResultTraced;
@@ -223,6 +220,8 @@ impl BuildRequestCommand {
             }),
             overrides,
             prompter: Box::new(CliPrompter),
+            show_sensitive: true,
+            root_dir: collection_file.parent().to_owned(),
             state: Default::default(),
         };
         let seed = RequestSeed::new(self.recipe_id, BuildOptions::default());
