@@ -12,7 +12,6 @@ use crate::{
     http::{Exchange, ExchangeSummary, RequestId},
 };
 use anyhow::{Context, anyhow};
-use derive_more::Display;
 use rusqlite::{Connection, DatabaseName, OptionalExtension, named_params};
 use serde::{Serialize, de::DeserializeOwned};
 use slumber_util::{ResultTraced, paths};
@@ -101,84 +100,137 @@ impl Database {
     }
 
     /// Get a list of all collections
-    pub fn collections(&self) -> anyhow::Result<Vec<PathBuf>> {
+    pub fn get_collections(&self) -> anyhow::Result<Vec<CollectionMetadata>> {
         self.connection()
-            .prepare("SELECT path FROM collections")?
-            .query_map([], |row| {
-                Ok(row.get::<_, CollectionPath>("path")?.into())
-            })
+            .prepare("SELECT * FROM collections")?
+            .query_map([], |row| row.try_into())
             .context("Error fetching collections")?
             .collect::<rusqlite::Result<Vec<_>>>()
             .context("Error extracting collection data")
+            .traced()
+    }
+
+    /// Get a collection's ID by its path. This will canonicalize the path to
+    /// ensure it matches what we've stored in the DB. Return an error if the
+    /// path isn't in the DB.
+    pub fn get_collection_id(
+        &self,
+        path: &Path,
+    ) -> anyhow::Result<CollectionId> {
+        // Convert to canonicalize and make serializable
+        let path = CollectionPath::try_from_path_maybe_missing(path)?;
+
+        self.connection()
+            .query_row(
+                "SELECT id FROM collections WHERE path = :path",
+                named_params! {":path": &path},
+                |row| row.get::<_, CollectionId>("id"),
+            )
+            .map_err(|err| match err {
+                rusqlite::Error::QueryReturnedNoRows => {
+                    // Use Display impl here because this will get shown in
+                    // CLI output
+                    anyhow!("Unknown collection `{path}`")
+                }
+                other => anyhow::Error::from(other)
+                    .context("Error fetching collection ID"),
+            })
+            .traced()
+    }
+
+    /// Get metadata about a collection from its ID
+    pub fn get_collection_metadata(
+        &self,
+        id: CollectionId,
+    ) -> anyhow::Result<CollectionMetadata> {
+        self.connection()
+            .query_row(
+                "SELECT * FROM collections WHERE id = :id",
+                named_params! {":id": id},
+                |row| CollectionMetadata::try_from(row),
+            )
+            .map_err(|err| match err {
+                rusqlite::Error::QueryReturnedNoRows => {
+                    anyhow!("Unknown collection `{id}`")
+                }
+                other => anyhow::Error::from(other)
+                    .context("Error fetching collection ID"),
+            })
+            .traced()
+    }
+
+    /// Delete a collection from the DB, including all requests and other rows
+    /// associated with it
+    pub fn delete_collection(
+        &self,
+        collection: CollectionId,
+    ) -> anyhow::Result<()> {
+        // Delete all rows referencing the collection before deleting the
+        // collection. It would be nice to use `ON DELETE CASCADE`, but you can
+        // only set that when the table is created. Sqlite doesn't allowing
+        // creating or modifying foreign keys on a table once it's created.
+        // Manually deleting everything is simpler than writing a huge migration
+        // just to make this automatic
+
+        let statements = [
+            "DELETE FROM requests_v2 WHERE collection_id = :id",
+            "DELETE FROM ui_state_v2 WHERE collection_id = :id",
+            "DELETE FROM collections WHERE id = :id",
+        ];
+
+        // Shitty try block!
+        (|| {
+            let mut connection = self.connection();
+            let tx = connection.transaction()?;
+            for statement in statements {
+                tx.prepare(statement)?
+                    .execute(named_params! {":id": collection})?;
+            }
+            tx.commit()?;
+            Ok::<(), anyhow::Error>(())
+        })()
+        .context(format!("Error deleting collection `{collection}`"))
+        .traced()
     }
 
     /// Migrate all data for one collection into another, deleting the source
     /// collection
     pub fn merge_collections(
         &self,
-        source: &Path,
-        target: &Path,
+        source: CollectionId,
+        target: CollectionId,
     ) -> anyhow::Result<()> {
-        fn get_collection_id(
-            connection: &Connection,
-            path: &Path,
-        ) -> anyhow::Result<CollectionId> {
-            // Convert to canonicalize and make serializable
-            let path: CollectionPath = path.try_into()?;
-
-            connection
-                .query_row(
-                    "SELECT id FROM collections WHERE path = :path",
-                    named_params! {":path": &path},
-                    |row| row.get::<_, CollectionId>("id"),
-                )
-                .map_err(|err| match err {
-                    rusqlite::Error::QueryReturnedNoRows => {
-                        // Use Display impl here because this will get shown in
-                        // CLI output
-                        anyhow!("Unknown collection `{path}`")
-                    }
-                    other => anyhow::Error::from(other)
-                        .context("Error fetching collection ID"),
-                })
-                .traced()
-        }
-
         info!(?source, ?target, "Merging database state");
-        let connection = self.connection();
-
-        // Exchange each path for an ID
-        let source = get_collection_id(&connection, source)?;
-        let target = get_collection_id(&connection, target)?;
+        let mut connection = self.connection();
+        let tx = connection.transaction()?;
 
         // Update each table in individually
-        connection
-            .execute(
-                "UPDATE requests_v2 SET collection_id = :target
+        tx.execute(
+            "UPDATE requests_v2 SET collection_id = :target
                 WHERE collection_id = :source",
-                named_params! {":source": source, ":target": target},
-            )
-            .context("Error migrating table `requests_v2`")
-            .traced()?;
-        connection
-            .execute(
-                // Overwrite UI state. Maybe this isn't the best UX, but sqlite
-                // doesn't provide an "UPDATE OR DELETE" so this is easiest and
-                // still reasonable
-                "UPDATE OR REPLACE ui_state_v2 SET collection_id = :target
+            named_params! {":source": source, ":target": target},
+        )
+        .context("Error migrating table `requests_v2`")
+        .traced()?;
+        tx.execute(
+            // Overwrite UI state. Maybe this isn't the best UX, but sqlite
+            // doesn't provide an "UPDATE OR DELETE" so this is easiest and
+            // still reasonable
+            "UPDATE OR REPLACE ui_state_v2 SET collection_id = :target
                 WHERE collection_id = :source",
-                named_params! {":source": source, ":target": target},
-            )
-            .context("Error migrating table `ui_state_v2`")
-            .traced()?;
+            named_params! {":source": source, ":target": target},
+        )
+        .context("Error migrating table `ui_state_v2`")
+        .traced()?;
+
+        tx.execute(
+            "DELETE FROM collections WHERE id = :source",
+            named_params! {":source": source},
+        )
+        .context("Error deleting source collection")
+        .traced()?;
+        tx.commit()?;
 
-        connection
-            .execute(
-                "DELETE FROM collections WHERE id = :source",
-                named_params! {":source": source},
-            )
-            .context("Error deleting source collection")
-            .traced()?;
         Ok(())
     }
 
@@ -221,7 +273,7 @@ impl Database {
         file: &CollectionFile,
     ) -> anyhow::Result<CollectionDatabase> {
         // Convert to canonicalize and make serializable
-        let path: CollectionPath = file.path().try_into()?;
+        let path = CollectionPath::try_from_path(file.path())?;
 
         // We have to set/get in two separate queries, because RETURNING doesn't
         // return anything if the insert didn't modify
@@ -263,18 +315,39 @@ pub struct CollectionDatabase {
 }
 
 impl CollectionDatabase {
-    /// Get the full path for the collection file associated with this DB handle
-    pub fn collection_path(&self) -> anyhow::Result<PathBuf> {
+    /// Get metadata for the collection associated with this DB handle
+    pub fn metadata(&self) -> anyhow::Result<CollectionMetadata> {
         self.database
             .connection()
             .query_row(
-                "SELECT path FROM collections WHERE id = :id",
+                "SELECT * FROM collections WHERE id = :id",
                 named_params! {":id": self.collection_id},
-                |row| row.get::<_, CollectionPath>("path"),
+                |row| row.try_into(),
             )
             .context("Error fetching collection path")
             .traced()
-            .map(PathBuf::from)
+    }
+
+    /// Get the root database, which has access to all collections
+    pub fn root(&self) -> &Database {
+        &self.database
+    }
+
+    /// Set the collection's display name. This should be set whenever the
+    /// collection file is loaded to ensure it's up to date in the database.
+    ///
+    /// If this fails it will log the result, but not return it. There's nothing
+    /// meaningful for the caller to do with the result beyond log it again.
+    pub fn set_name(&self, name: Option<&str>) {
+        let _ = self
+            .database
+            .connection()
+            .execute(
+                "UPDATE collections SET name = :name WHERE id = :id",
+                named_params! {":id": self.collection_id, ":name": name},
+            )
+            .context("Error updating collection name")
+            .traced();
     }
 
     /// Get a request by ID, or `None` if it does not exist in history.
@@ -491,8 +564,7 @@ impl CollectionDatabase {
         recipe_id: &RecipeId,
     ) -> anyhow::Result<usize> {
         info!(
-            collection_id = %self.collection_id,
-            collection_path = ?self.collection_path(),
+            collection = ?self.metadata(),
             %recipe_id,
             ?profile_id,
             "Deleting all requests for recipe+profile",
@@ -522,8 +594,7 @@ impl CollectionDatabase {
     /// Delete a single request by ID
     pub fn delete_request(&self, request_id: RequestId) -> anyhow::Result<()> {
         info!(
-            collection_id = %self.collection_id,
-            collection_path = ?self.collection_path(),
+            collection = ?self.metadata(),
             %request_id,
             "Deleting request"
         );
@@ -605,7 +676,7 @@ impl CollectionDatabase {
         Ok(())
     }
 
-    #[cfg(test)]
+    /// Get the unique ID of this collection
     pub fn collection_id(&self) -> CollectionId {
         self.collection_id
     }
@@ -613,8 +684,10 @@ impl CollectionDatabase {
 
 /// A unique ID for a collection. This is generated when the collection is
 /// inserted into the DB.
-#[derive(Copy, Clone, Debug, Display)]
-#[cfg_attr(test, derive(Eq, Hash, PartialEq))]
+#[derive(
+    Copy, Clone, Debug, derive_more::Display, derive_more::FromStr, PartialEq,
+)]
+#[cfg_attr(any(test, feature = "test"), derive(Eq, Hash))]
 pub struct CollectionId(Uuid);
 
 impl CollectionId {
@@ -623,6 +696,24 @@ impl CollectionId {
     }
 }
 
+/// Info about a collection from the database
+#[derive(Clone, Debug)]
+#[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
+pub struct CollectionMetadata {
+    pub id: CollectionId,
+    pub path: PathBuf,
+    pub name: Option<String>,
+}
+
+impl CollectionMetadata {
+    /// Get the name if available, otherwise fall back to the path
+    pub fn display_name(&self) -> String {
+        self.name
+            .clone()
+            .unwrap_or_else(|| self.path.display().to_string())
+    }
+}
+
 #[cfg(any(test, feature = "test"))]
 impl slumber_util::Factory for Database {
     fn factory((): ()) -> Self {
