@@ -5,23 +5,26 @@
 mod functions;
 #[cfg(test)]
 mod tests;
+mod util;
 
 #[cfg(any(test, feature = "test"))]
 use crate::collection::Recipe;
 use crate::{
     collection::{Collection, Profile, ProfileId, RecipeId},
     http::{Exchange, RequestSeed, ResponseRecord, TriggeredRequestError},
-    render::functions::RequestTrigger,
-    util::{FutureCache, FutureCacheOutcome},
+    render::{
+        functions::RequestTrigger,
+        util::{FieldCache, FieldCacheOutcome},
+    },
 };
 use anyhow::anyhow;
 use async_trait::async_trait;
 use chrono::Utc;
-use derive_more::From;
+use derive_more::{Deref, From};
 use indexmap::IndexMap;
 use itertools::Itertools;
 use serde_json_path::JsonPath;
-use slumber_template::{Arguments, Identifier, RenderError};
+use slumber_template::{Arguments, Identifier, LazyValue, RenderError};
 use slumber_util::ResultTraced;
 use std::{
     fmt::Debug, io, iter, path::PathBuf, process::ExitStatus, sync::Arc,
@@ -33,6 +36,11 @@ use tokio::sync::oneshot;
 /// templating. Unfortunately this has to own all data so templating can be
 /// deferred into a task (tokio requires `'static` for spawned tasks). If this
 /// becomes a bottleneck, we can `Arc` some stuff.
+///
+/// One instance of this context applies to all renders in a group (a render
+/// group is all the renders for a single request). [SingleRenderContext] is a
+/// wrapper for each individual render. Use [Self::streaming] to get the wrapper
+/// for a render.
 #[derive(Debug)]
 pub struct TemplateContext {
     /// Entire request collection
@@ -59,6 +67,15 @@ pub struct TemplateContext {
 }
 
 impl TemplateContext {
+    ///  Wrap this context for a single render, with streaming optionally
+    /// enabled
+    pub fn streaming(&self, can_stream: bool) -> SingleRenderContext<'_> {
+        SingleRenderContext {
+            context: self,
+            can_stream,
+        }
+    }
+
     fn current_profile(&self) -> Option<&Profile> {
         self.selected_profile
             .as_ref()
@@ -82,76 +99,105 @@ impl TemplateContext {
             });
         }
 
-        // Defer loading the most recent exchange until we know we'll need it
-        let get_latest = || async {
-            self.http_provider
-                .get_latest_request(self.selected_profile.as_ref(), recipe_id)
-                .await
-                .map_err(FunctionError::Database)
-        };
-
-        // Helper to execute the request, if triggered
-        let send_request = || async {
-            // There are 3 different ways we can generate the build optoins:
-            // 1. Default (enable all query params/headers)
-            // 2. Load from UI state for both TUI and CLI
-            // 3. Load from UI state for TUI, enable all for CLI
-            // These all have their own issues:
-            // 1. Triggered request doesn't necessarily match behavior if user
-            //  were to execute the request themself
-            // 2. CLI behavior is silently controlled by UI state
-            // 3. TUI and CLI behavior may not match
-            // All 3 options are unintuitive in some way, but 1 is the easiest
-            // to implement so I'm going with that for now.
-            let build_options = Default::default();
-
-            self.http_provider
-                .send_request(
-                    RequestSeed::new(recipe_id.clone(), build_options),
-                    self,
-                )
-                .await
-                .map_err(|error| FunctionError::Trigger {
-                    recipe_id: recipe_id.clone(),
-                    error,
-                })
-        };
-
         let exchange = match trigger {
-            RequestTrigger::Never => {
-                get_latest().await?.ok_or(FunctionError::ResponseMissing)?
-            }
+            RequestTrigger::Never => self
+                .get_latest_cached(recipe_id)
+                .await?
+                .ok_or(FunctionError::ResponseMissing)?,
             RequestTrigger::NoHistory => {
                 // If a exchange is present in history, use that. If not, fetch
-                if let Some(exchange) = get_latest().await? {
-                    exchange
-                } else {
-                    send_request().await?
-                }
-            }
-            RequestTrigger::Expire { duration } => match get_latest().await? {
-                Some(exchange)
-                    if exchange.end_time + duration.inner() >= Utc::now() =>
+                if let Some(exchange) =
+                    self.get_latest_cached(recipe_id).await?
                 {
                     exchange
+                } else {
+                    self.send_request(recipe_id).await?
                 }
-                _ => send_request().await?,
-            },
-            RequestTrigger::Always => send_request().await?,
+            }
+            RequestTrigger::Expire { duration } => {
+                match self.get_latest_cached(recipe_id).await? {
+                    Some(exchange)
+                        if exchange.end_time + duration.inner()
+                            >= Utc::now() =>
+                    {
+                        exchange
+                    }
+                    _ => self.send_request(recipe_id).await?,
+                }
+            }
+            RequestTrigger::Always => self.send_request(recipe_id).await?,
         };
 
         Ok(exchange.response)
     }
+
+    /// Get the most recent cached exchange for the given recipe
+    async fn get_latest_cached(
+        &self,
+        recipe_id: &RecipeId,
+    ) -> Result<Option<Exchange>, FunctionError> {
+        self.http_provider
+            .get_latest_request(self.selected_profile.as_ref(), recipe_id)
+            .await
+            .map_err(FunctionError::Database)
+    }
+
+    /// Send a request for the recipe and return the exchange
+    async fn send_request(
+        &self,
+        recipe_id: &RecipeId,
+    ) -> Result<Exchange, FunctionError> {
+        // There are 3 different ways we can generate the build optoins:
+        // 1. Default (enable all query params/headers)
+        // 2. Load from UI state for both TUI and CLI
+        // 3. Load from UI state for TUI, enable all for CLI
+        // These all have their own issues:
+        // 1. Triggered request doesn't necessarily match behavior if user
+        //  were to execute the request themself
+        // 2. CLI behavior is silently controlled by UI state
+        // 3. TUI and CLI behavior may not match
+        // All 3 options are unintuitive in some way, but 1 is the easiest
+        // to implement so I'm going with that for now.
+        let build_options = Default::default();
+
+        self.http_provider
+            .send_request(
+                RequestSeed::new(recipe_id.clone(), build_options),
+                self,
+            )
+            .await
+            .map_err(|error| FunctionError::Trigger {
+                recipe_id: recipe_id.clone(),
+                error,
+            })
+    }
 }
 
-impl slumber_template::Context for TemplateContext {
-    async fn get(
+/// A wrapper for [TemplateContext] that provides the [Context] trait. While
+/// [TemplateContext] is intended to be used for multiple renders within a
+/// render group, this is meant for an individual render. As such, it captures
+/// settings that can vary across different renders in the same group.
+#[derive(Debug, Deref)]
+pub struct SingleRenderContext<'a> {
+    #[deref]
+    context: &'a TemplateContext,
+    /// Is streaming supported for this component? Enabled for request bodies,
+    /// disabled for everything else.
+    can_stream: bool,
+}
+
+impl slumber_template::Context for SingleRenderContext<'_> {
+    fn can_stream(&self) -> bool {
+        self.can_stream
+    }
+
+    async fn get_field(
         &self,
-        field: &slumber_template::Identifier,
-    ) -> Result<slumber_template::Value, slumber_template::RenderError> {
+        field: &Identifier,
+    ) -> Result<LazyValue, RenderError> {
         // Check overrides first. The override value is NOT treated as a
         // template
-        if let Some(value) = self.overrides.get(field.as_str()) {
+        if let Some(value) = self.context.overrides.get(field.as_str()) {
             return Ok(value.clone().into());
         }
 
@@ -160,26 +206,20 @@ impl slumber_template::Context for TemplateContext {
         // If not, we get a guard back, meaning we're responsible for the
         // computation. At the end, we'll write back to the guard so everyone
         // else can copy our homework.
-        let cache = &self.state.profile_results;
-        let guard = match cache.get_or_init(field.clone()).await {
-            FutureCacheOutcome::Hit(value) => return Ok(value),
-            FutureCacheOutcome::Miss(guard) => guard,
-            // The future responsible for writing to the guard failed. All the
-            // places that render multiple templates use try_join so one failure
-            // should cause all other futures to stop being polled. So
-            // theoretically this error will never be seen, but we return an
-            // error instead of panicking just to be safe. We could clone the
-            // error and share it between calls, but implementing Clone for an
-            // error type can be pretty annoying.
-            FutureCacheOutcome::NoResponse => {
-                return Err(RenderError::CacheFailed {
-                    field: field.clone(),
-                });
-            }
+        let guard = match self
+            .context
+            .state
+            .field_cache
+            .get_or_init(field.clone())
+            .await
+        {
+            FieldCacheOutcome::Hit(value) => return Ok(value.into()),
+            FieldCacheOutcome::Miss(guard) => guard,
         };
 
-        // Then check the current profile
+        // We're responsible for the computation. Grab the field's value
         let template = self
+            .context
             .current_profile()
             .and_then(|profile| profile.data.get(field.as_str()))
             .ok_or_else(|| FunctionError::UnknownField {
@@ -187,32 +227,48 @@ impl slumber_template::Context for TemplateContext {
             })?;
 
         // Render the nested template
-        let bytes = template.render_bytes(self).await.map_err(|error| {
-            FunctionError::ProfileNested {
-                field: field.clone(),
-                error,
-            }
-        })?;
-        let value: slumber_template::Value = bytes.into();
+        let output = template.render(self).await;
 
-        // Store value in the cache so other instances of this chain can use it
-        guard.set(value.clone());
-        Ok(value)
+        // If the output is a value, we can cache it. If it's a stream, it can't
+        // be cloned so it can't be cached. In practice there's probably no
+        // reason to include the same stream field twice in a single body, but
+        // if that happens we'll have to compute it twice. This saves us a lot
+        // of annoying machinery though.
+        if output.has_stream() {
+            // If the nested template rendered to a single chunk, we can unpack
+            // it out of its chunk list. If it had multiple chunks, we need to
+            // keep all of them to provide both a correct preview and the final
+            // stream
+            Ok(output.unpack())
+        } else {
+            let value = output.try_collect_value().await.map_err(
+                // We *could* just return the error, but wrap it to give
+                // additional context
+                |error| {
+                    RenderError::from(FunctionError::ProfileNested {
+                        field: field.clone(),
+                        error,
+                    })
+                },
+            )?;
+            guard.set(value.clone());
+            Ok(LazyValue::Value(value))
+        }
     }
 
     async fn call(
         &self,
         function_name: &Identifier,
         arguments: Arguments<'_, Self>,
-    ) -> Result<slumber_template::Value, RenderError> {
+    ) -> Result<LazyValue, RenderError> {
         match function_name.as_str() {
             "base64" => functions::base64(arguments),
             "boolean" => functions::boolean(arguments),
-            "command" => functions::command(arguments).await,
+            "command" => functions::command(arguments),
             "concat" => functions::concat(arguments),
             "debug" => functions::debug(arguments),
             "env" => functions::env(arguments),
-            "file" => functions::file(arguments).await,
+            "file" => functions::file(arguments),
             "float" => functions::float(arguments),
             "integer" => functions::integer(arguments),
             "json_parse" => functions::json_parse(arguments),
@@ -256,7 +312,7 @@ impl
             database::CollectionDatabase,
             test_util::{TestHttpProvider, TestPrompter},
         };
-        use slumber_util::paths::get_repo_root;
+        use slumber_util::test_data_dir;
 
         let selected_profile = profiles.first().map(|(id, _)| id.clone());
         Self {
@@ -273,7 +329,7 @@ impl
             )),
             overrides: IndexMap::new(),
             prompter: Box::<TestPrompter>::default(),
-            root_dir: get_repo_root().to_owned(),
+            root_dir: test_data_dir(),
             show_sensitive: true,
             state: Default::default(),
         }
@@ -290,7 +346,7 @@ pub struct RenderGroupState {
     /// times. If a field fails to render, the guard holder should drop the
     /// guard without entering a result. This will kill the entire render so
     /// other renderers of that field will be cancelled.
-    profile_results: FutureCache<Identifier, slumber_template::Value>,
+    field_cache: FieldCache,
 }
 
 /// An abstraction that provides behavior for chained HTTP requests. This
@@ -385,32 +441,26 @@ pub enum FunctionError {
     #[error(transparent)]
     Base64Decode(#[from] base64::DecodeError),
 
-    /// Error creating or spawning a subprocess
+    /// Error creating, spawning, or executing a subprocess
     #[error(
-        "Executing command `{}`", iter::once(program).chain(args).format(" ")
+        "Executing command `{}`", iter::once(program).chain(arguments).format(" ")
     )]
     CommandInit {
         program: String,
-        args: Vec<String>,
+        arguments: Vec<String>,
         #[source]
         error: io::Error,
     },
 
     /// Command exited with a non-zero status code
     #[error(
-        "Command `{command}` exited with {status}\n{stderr}",
-        command = iter::once(program).chain(args).format(" "),
-        stderr = String::from_utf8_lossy(stderr),
+        "Command `{command}` exited with {status}",
+        command = iter::once(program).chain(arguments).format(" "),
     )]
     CommandStatus {
         program: String,
-        args: Vec<String>,
+        arguments: Vec<String>,
         status: ExitStatus,
-        // Storing stdout+stderr because I like symmetry. It's not easy to
-        // print both because we don't know how they're supposed to be
-        // interleaved. The error should be in stderr so we'll just print that.
-        stdout: Vec<u8>,
-        stderr: Vec<u8>,
     },
 
     /// User passed an empty command arrary
