@@ -11,13 +11,16 @@ use crate::{
         http_engine,
     },
 };
+use bytes::BytesMut;
 use chrono::{DateTime, Utc};
+use futures::{StreamExt, TryStreamExt};
 use indexmap::{IndexMap, indexmap};
 use rstest::rstest;
 use serde_json::json;
-use slumber_template::{Expression, Literal, Template};
+use slumber_template::{Expression, Literal, StreamSource, Template, Value};
 use slumber_util::{
-    Factory, TempDir, assert_result, paths::get_repo_root, temp_dir,
+    Factory, TempDir, assert_matches, assert_result, paths::get_repo_root,
+    temp_dir,
 };
 use std::time::Duration;
 use tokio::fs;
@@ -40,7 +43,10 @@ async fn test_profile() {
     let context = TemplateContext::factory((by_id([profile]), IndexMap::new()));
 
     assert_eq!(
-        template.render_bytes(&context).await.unwrap(),
+        template
+            .render_bytes(&context.streaming(false))
+            .await
+            .unwrap(),
         "http://localhost/users/1"
     );
 }
@@ -65,7 +71,10 @@ async fn test_override() {
     };
 
     assert_eq!(
-        template.render_bytes(&context).await.unwrap(),
+        template
+            .render_bytes(&context.streaming(false))
+            .await
+            .unwrap(),
         "http://override/users/1"
     );
 }
@@ -89,7 +98,9 @@ async fn test_base64(
         [("decode", Some(decode.into()))],
     );
     assert_result(
-        template.render_bytes(&TemplateContext::factory(())).await,
+        template
+            .render_bytes(&TemplateContext::factory(()).streaming(false))
+            .await,
         expected,
     );
 }
@@ -113,8 +124,12 @@ async fn test_base64(
 async fn test_boolean(#[case] input: Expression, #[case] expected: bool) {
     let template = Template::function_call("boolean", [input], []);
     assert_result(
-        template.render_value(&TemplateContext::factory(())).await,
-        Ok(slumber_template::Value::Boolean(expected)),
+        template
+            .render(&TemplateContext::factory(()).streaming(false))
+            .await
+            .try_collect_value()
+            .await,
+        Ok(Value::Boolean(expected)),
     );
 }
 
@@ -126,15 +141,15 @@ async fn test_boolean(#[case] input: Expression, #[case] expected: bool) {
 // code, there isn't much value in testing all platforms.
 #[cfg_attr(
     unix,
-    case::root_dir(vec!["pwd"], None, None, Ok("{ROOT}\n".as_bytes())),
+    case::root_dir(vec!["pwd"], None, None, Ok("{ROOT}/test_data\n".as_bytes())),
 )]
 #[cfg_attr(
     unix,
     case::cwd(
         vec!["pwd"],
-        Some("test_data"),
+        Some(".."), // We start in the test_data dir
         None,
-        Ok("{ROOT}/test_data\n".as_bytes()),
+        Ok("{ROOT}\n".as_bytes()),
     ),
 )]
 #[case::stdin(
@@ -162,8 +177,7 @@ async fn test_boolean(#[case] input: Expression, #[case] expected: bool) {
     vec!["ls", "--fake"],
     None,
     None,
-    // Error message varies by platform
-    Err(if cfg!(unix) { "unrecognized option" } else { "unknown option" }),
+    Err("Command `ls --fake` exited with"),
 )]
 #[tokio::test]
 async fn test_command(
@@ -180,21 +194,53 @@ async fn test_command(
             ("stdin", stdin.map(Expression::from)),
         ],
     );
-    let root_dir = get_repo_root();
-    let context = TemplateContext {
-        root_dir: root_dir.to_owned(),
-        ..TemplateContext::factory(())
-    };
     // Replace {ROOT} with the root dir
     let expected = expected.map(|bytes| {
         if let Ok(s) = std::str::from_utf8(bytes) {
-            s.replace("{ROOT}", &root_dir.to_string_lossy())
+            s.replace("{ROOT}", &get_repo_root().to_string_lossy())
                 .into_bytes()
         } else {
             bytes.to_owned()
         }
     });
-    assert_result(template.render_bytes(&context).await, expected);
+    assert_result(
+        template
+            .render_bytes(&TemplateContext::factory(()).streaming(false))
+            .await,
+        expected,
+    );
+}
+
+/// Test that the command isn't spawned until the output stream is evaluated.
+/// This ensures we don't execute the command for previews
+#[tokio::test]
+async fn test_command_lazy() {
+    let template = Template::function_call(
+        "command",
+        [vec!["i-will-fail".into()].into()],
+        [],
+    );
+    // This shouldn't fail because the command isn't evaluated yet
+    let output = template
+        .render(&TemplateContext::factory(()).streaming(true))
+        .await;
+    assert_eq!(
+        output.stream_source(),
+        Some(&StreamSource::Command {
+            command: vec!["i-will-fail".into()]
+        })
+    );
+    let stream = output.try_into_stream().unwrap();
+
+    // Error happens when we collect
+    assert_result(
+        stream.try_collect::<BytesMut>().await,
+        Err::<BytesMut, &str>(if cfg!(unix) {
+            "No such file or directory"
+        } else {
+            "program not found"
+        }),
+    );
 }
 
 /// `concat()`
@@ -212,7 +258,9 @@ async fn test_concat(
 ) {
     let template = Template::function_call("concat", [elements.into()], []);
     assert_result(
-        template.render_string(&TemplateContext::factory(())).await,
+        template
+            .render_string(&TemplateContext::factory(()).streaming(false))
+            .await,
         expected,
     );
 }
@@ -228,7 +276,9 @@ async fn test_env(
 ) {
     let template = Template::function_call("env", [variable.into()], []);
     assert_result(
-        template.render_bytes(&TemplateContext::factory(())).await,
+        template
+            .render_bytes(&TemplateContext::factory(()).streaming(false))
+            .await,
         expected,
     );
 }
@@ -264,7 +314,10 @@ async fn test_file(
         ..TemplateContext::factory(())
     };
 
-    assert_result(template.render_bytes(&context).await, expected);
+    assert_result(
+        template.render_bytes(&context.streaming(false)).await,
+        expected,
+    );
 }
 
 /// Bonus test case for ~ expansion in file(). Only test on Linux because
@@ -285,7 +338,10 @@ async fn test_file_tilde(temp_dir: TempDir) {
 
     let guard =
         env_lock::lock_env([("HOME", Some(temp_dir.to_str().unwrap()))]);
-    assert_result(template.render_string(&context).await, Ok("text"));
+    assert_result(
+        template.render_string(&context.streaming(false)).await,
+        Ok("text"),
+    );
     drop(guard);
 }
 
@@ -312,8 +368,12 @@ async fn test_float(
 ) {
     let template = Template::function_call("float", [input], []);
     assert_result(
-        template.render_value(&TemplateContext::factory(())).await,
-        expected.map(slumber_template::Value::from),
+        template
+            .render(&TemplateContext::factory(()).streaming(false))
+            .await
+            .try_collect_value()
+            .await,
+        expected.map(Value::from),
     );
 }
 
@@ -340,8 +400,12 @@ async fn test_integer(
 ) {
     let template = Template::function_call("integer", [input], []);
     assert_result(
-        template.render_value(&TemplateContext::factory(())).await,
-        expected.map(slumber_template::Value::from),
+        template
+            .render(&TemplateContext::factory(()).streaming(false))
+            .await
+            .try_collect_value()
+            .await,
+        expected.map(Value::from),
     );
 }
 
@@ -355,11 +419,15 @@ async fn test_integer(
 #[tokio::test]
 async fn test_json_parse(
     #[case] json: &'static [u8],
-    #[case] expected: Result<slumber_template::Value, &str>,
+    #[case] expected: Result<Value, &str>,
 ) {
     let template = Template::function_call("json_parse", [json.into()], []);
     assert_result(
-        template.render_value(&TemplateContext::factory(())).await,
+        template
+            .render(&TemplateContext::factory(()).streaming(false))
+            .await
+            .try_collect_value()
+            .await,
         expected,
     );
 }
@@ -419,7 +487,9 @@ async fn test_jsonpath(
         [("mode", mode.map(Expression::from))],
     );
     assert_result(
-        template.render_string(&TemplateContext::factory(())).await,
+        template
+            .render_string(&TemplateContext::factory(()).streaming(false))
+            .await,
         expected,
     );
 }
@@ -453,7 +523,10 @@ async fn test_prompt(
         show_sensitive: false,
         ..TemplateContext::factory(())
     };
-    assert_result(template.render_bytes(&context).await, expected);
+    assert_result(
+        template.render_bytes(&context.streaming(false)).await,
+        expected,
+    );
 }
 
 /// `response()`
@@ -619,7 +692,10 @@ async fn test_response(
         ..TemplateContext::factory((IndexMap::new(), by_id(recipes)))
     };
 
-    assert_result(template.render_bytes(&context).await, expected);
+    assert_result(
+        template.render_bytes(&context.streaming(false)).await,
+        expected,
+    );
 }
 
 /// `response_header()`. We're leaning on the `response()` tests for most of
@@ -685,7 +761,10 @@ async fn test_response_header(
         ..TemplateContext::factory((IndexMap::new(), by_id([recipe])))
     };
 
-    assert_result(template.render_bytes(&context).await, expected);
+    assert_result(
+        template.render_bytes(&context.streaming(false)).await,
+        expected,
+    );
 }
 
 /// `select()`
@@ -708,7 +787,10 @@ async fn test_select(
         prompter: Box::new(TestSelectPrompter::new(select.into_iter())),
         ..TemplateContext::factory(())
     };
-    assert_result(template.render_bytes(&context).await, expected);
+    assert_result(
+        template.render_bytes(&context.streaming(false)).await,
+        expected,
+    );
 }
 
 /// `sensitive()`
@@ -721,7 +803,13 @@ async fn test_sensitive(#[case] input: &str, #[case] expected: &str) {
         show_sensitive: false,
         ..TemplateContext::factory(())
     };
-    assert_eq!(template.render_bytes(&context).await.unwrap(), expected);
+    assert_eq!(
+        template
+            .render_bytes(&context.streaming(false))
+            .await
+            .unwrap(),
+        expected
+    );
 }
 
 /// `string()`
@@ -738,7 +826,9 @@ async fn test_string(
 ) {
     let template = Template::function_call("string", [input], []);
     assert_result(
-        template.render_string(&TemplateContext::factory(())).await,
+        template
+            .render_string(&TemplateContext::factory(()).streaming(false))
+            .await,
         expected,
     );
 }
@@ -762,13 +852,114 @@ async fn test_trim(
     );
     assert_eq!(
         template
-            .render_bytes(&TemplateContext::factory(()))
+            .render_bytes(&TemplateContext::factory(()).streaming(false))
             .await
             .unwrap(),
         expected
     );
 }
 
+/// Test that the stream source is retained for a single-chunk template
+#[rstest]
+#[case::stream_root("{{ file('data.json') }}", true)]
+#[case::stream_piped("{{ 'data.json' | file() }}", true)]
+#[case::stream_via_profile("{{ file_field }}", true)]
+// Multiple chunks means we don't have a single stream source
+#[case::no_stream_not_root("data: {{ file('data.json') }}", false)]
+#[case::no_stream_not_root_via_profile("data: {{ file_field }}", false)]
+#[tokio::test]
+async fn test_stream_source(
+    #[case] template: Template,
+    #[case] expected_has_source: bool,
+) {
+    // Put some profile data in the context
+    let profile_data = indexmap! {
+        "file_field".into() => "{{ file('data.json') }}".into(),
+    };
+    let profile = Profile {
+        data: profile_data,
+        ..Profile::factory(())
+    };
+    let context = TemplateContext::factory((by_id([profile]), IndexMap::new()));
+
+    let output = template.render(&context.streaming(true)).await;
+    if expected_has_source {
+        assert_matches!(output.stream_source(), Some(_));
+    } else {
+        assert_matches!(output.stream_source(), None);
+    }
+}
+
+/// Test that streamed templates are actually computed chunk-by-chunk and are
+/// never eagerly collected
+#[rstest]
+#[tokio::test]
+async fn test_stream_chunks(temp_dir: TempDir) {
+    // Put some profile data in the context
+    let profile_data = indexmap! {
+        "one_chunk".into() => "{{ file('second') }}".into(),
+        // This one has multiple chunks. Need to make sure this doesn't get
+        // collected, and the whole stream is passed through
+        "multi_chunk".into() => "{{ file('third') }} | {{ file('fourth') }}".into(),
+    };
+    let profile = Profile {
+        data: profile_data,
+        ..Profile::factory(())
+    };
+    let context = TemplateContext {
+        root_dir: temp_dir.to_owned(),
+        ..TemplateContext::factory((by_id([profile]), IndexMap::new()))
+    };
+
+    // Testing that streaming directly or via a profile field loads the value
+    // lazily
+    let template = Template::from(
+        "{{ file('first') }} | {{ one_chunk }} | {{ multi_chunk }}",
+    );
+
+    // Stream init succeeds even though the files don't exist yet, because they
+    // aren't loaded until the respective chunk is loaded
+    let mut stream = template
+        .render(&context.streaming(true))
+        .await
+        .try_into_stream()
+        .unwrap();
+    // Convert chunks to strings for better assertions
+    let mut next_chunk = async move || {
+        stream.next().await.map(|result| {
+            let bytes = result.unwrap();
+            String::from_utf8(bytes.into()).unwrap()
+        })
+    };
+
+    let write_file = async |name: &str| {
+        fs::write(temp_dir.join(name), name).await.unwrap();
+    };
+    // Stream should be 3 chunks. Each chunk isn't computed until requested.
+    // By creating the files right before reading, we assure that it isn't
+    // loaded until the chunk is requested
+    write_file("first").await;
+    assert_eq!(next_chunk().await.as_deref(), Some("first"));
+
+    assert_eq!(next_chunk().await.as_deref(), Some(" | "));
+
+    // Profile field with one chunk
+    write_file("second").await;
+    assert_eq!(next_chunk().await.as_deref(), Some("second"));
+
+    assert_eq!(next_chunk().await.as_deref(), Some(" | "));
+
+    // Profile field with multiple chunks
+    write_file("third").await;
+    assert_eq!(next_chunk().await.as_deref(), Some("third"));
+    assert_eq!(next_chunk().await.as_deref(), Some(" | "));
+    write_file("fourth").await;
+    assert_eq!(next_chunk().await.as_deref(), Some("fourth"));
+
+    // Job's done
+    assert_eq!(next_chunk().await, None);
+}
+
 /// Bytes that can't be converted to a string
 fn invalid_utf8() -> &'static [u8] {
     b"\xc3\x28"
