@@ -8,8 +8,11 @@ pub struct HeaderTable<'a> {
     pub headers: &'a HeaderMap,
 }
 
-impl<'a> Generate for HeaderTable<'a> {
-    type Output<'this> = ratatui::widgets::Table<'this> where Self: 'this;
+impl Generate for HeaderTable<'_> {
+    type Output<'this>
+        = ratatui::widgets::Table<'this>
+    where
+        Self: 'this;
 
     fn generate<'this>(self) -> Self::Output<'this>
     where
