@@ -121,7 +121,7 @@ impl QueryableBody {
     }
 
     /// Get whatever text the user sees
-    pub fn visible_text(&self) -> &Text {
+    pub fn visible_text(&self) -> &Text<'_> {
         &self.text_state.text
     }
 
@@ -400,14 +400,16 @@ impl TextState {
             // separate task because it's generally very fast. If this is slow
             // enough that it affects the user, the "large" body size is
             // probably too low
-            // unstable: if-let chain
-            // https://github.com/rust-lang/rust/pull/132833
-            let (text, pretty): (Cow<str>, bool) = match content_type {
-                Some(content_type) if prettify => content_type
+            let (text, pretty): (Cow<str>, bool) = if let Some(content_type) =
+                content_type
+                && prettify
+            {
+                content_type
                     .prettify(text)
                     .map(|body| (Cow::Owned(body), true))
-                    .unwrap_or((Cow::Borrowed(text), false)),
-                _ => (Cow::Borrowed(text), false),
+                    .unwrap_or((Cow::Borrowed(text), false))
+            } else {
+                (Cow::Borrowed(text), false)
             };
 
             let text =
@@ -479,13 +481,13 @@ mod tests {
             util::persistence::{DatabasePersistedStore, PersistedLazy},
         },
     };
-    use crossterm::event::KeyCode;
     use persisted::{PersistedKey, PersistedStore};
     use ratatui::{layout::Margin, text::Span};
     use rstest::{fixture, rstest};
     use serde::Serialize;
     use slumber_core::http::{ResponseBody, ResponseRecord};
     use slumber_util::{Factory, TempDir, assert_matches, temp_dir};
+    use terminput::KeyCode;
     use tokio::fs;
 
     const TEXT: &str = "{\"greeting\":\"hello\"}";
@@ -496,7 +498,7 @@ mod tests {
     struct Key;
 
     /// Style text to match the text window gutter
-    fn gutter(text: &str) -> Span {
+    fn gutter(text: &str) -> Span<'_> {
         let styles = &TuiContext::get().styles;
         Span::styled(text, styles.text_window.gutter)
     }
