@@ -18,6 +18,7 @@ use crate::{
     },
 };
 use anyhow::Context;
+use indexmap::IndexMap;
 use mime::Mime;
 use ratatui::Frame;
 use serde::Serialize;
@@ -50,7 +51,7 @@ impl RecipeBodyDisplay {
     /// body is not `None`.
     pub fn new(body: &RecipeBody, recipe: &Recipe) -> Self {
         match body {
-            RecipeBody::Raw(body) => {
+            RecipeBody::Raw(body) | RecipeBody::Stream(body) => {
                 Self::Raw(TextBody::new(body.clone(), recipe).into())
             }
             RecipeBody::Json(json) => {
@@ -68,28 +69,38 @@ impl RecipeBodyDisplay {
                 let template = json_string.parse().unwrap();
                 Self::Json(TextBody::new(template, recipe).into())
             }
-            RecipeBody::FormUrlencoded(fields)
-            | RecipeBody::FormMultipart(fields) => {
-                let inner = RecipeFieldTable::new(
-                    "Field",
-                    FormRowKey(recipe.id.clone()),
-                    fields.iter().enumerate().map(|(i, (field, value))| {
-                        (
-                            field.clone(),
-                            value.clone(),
-                            RecipeOverrideKey::form_field(recipe.id.clone(), i),
-                            FormRowToggleKey {
-                                recipe_id: recipe.id.clone(),
-                                field: field.clone(),
-                            },
-                        )
-                    }),
-                );
-                Self::Form(inner.into())
+            RecipeBody::FormUrlencoded(fields) => {
+                Self::Form(Self::form_table(&recipe.id, fields, false).into())
+            }
+            RecipeBody::FormMultipart(fields) => {
+                Self::Form(Self::form_table(&recipe.id, fields, true).into())
             }
         }
     }
 
+    fn form_table(
+        recipe_id: &RecipeId,
+        fields: &IndexMap<String, Template>,
+        can_stream: bool,
+    ) -> RecipeFieldTable<FormRowKey, FormRowToggleKey> {
+        RecipeFieldTable::new(
+            "Field",
+            FormRowKey(recipe_id.clone()),
+            fields.iter().enumerate().map(|(i, (field, value))| {
+                (
+                    field.clone(),
+                    value.clone(),
+                    RecipeOverrideKey::form_field(recipe_id.clone(), i),
+                    FormRowToggleKey {
+                        recipe_id: recipe_id.clone(),
+                        field: field.clone(),
+                    },
+                )
+            }),
+            can_stream,
+        )
+    }
+
     /// If the user has applied a temporary edit to the body, get the override
     /// value. Return `None` to use the recipe's stock body.
     pub fn override_value(&self) -> Option<RecipeBody> {
@@ -177,6 +188,7 @@ impl TextBody {
                 RecipeOverrideKey::body(recipe.id.clone()),
                 template,
                 content_type,
+                true,
             ),
             mime,
             text_window: Component::default(),
