@@ -9,13 +9,13 @@ use crate::{
             modal::{Modal, ModalQueue},
         },
         component::{
-            help::HelpFooter,
+            footer::Footer,
             history::History,
-            misc::{ConfirmModal, NotificationText},
+            misc::ConfirmModal,
             primary::{PrimaryView, PrimaryViewProps},
         },
         context::UpdateContext,
-        draw::{Draw, DrawMetadata, Generate},
+        draw::{Draw, DrawMetadata},
         event::{Child, Event, EventHandler, OptionEvent},
         util::persistence::PersistedLazy,
     },
@@ -41,7 +41,7 @@ pub struct Root {
     // ==== Children =====
     primary_view: Component<PrimaryView>,
     modal_queue: Component<ModalQueue>,
-    notification_text: Component<Option<NotificationText>>,
+    footer: Component<Footer>,
 }
 
 impl Root {
@@ -58,7 +58,7 @@ impl Root {
             // Children
             primary_view: primary_view.into(),
             modal_queue: Component::default(),
-            notification_text: Component::default(),
+            footer: Component::default(),
         }
     }
 
@@ -163,29 +163,30 @@ impl EventHandler for Root {
                     // Walk down the component tree and collect actions from
                     // all visible+focused components
                     let actions = self.primary_view.collect_actions();
-                    ActionsModal::new(actions).open();
+                    // Actions can be empty if a modal is already open
+                    if !actions.is_empty() {
+                        ActionsModal::new(actions).open();
+                    }
                 }
                 Action::History => {
                     self.open_history(context.request_store)
                         .reported(&ViewContext::messages_tx());
                 }
                 Action::Cancel => {
-                    if let Some(request_id) = self.selected_request_id.0 {
-                        // unstable: if-let chain
-                        // https://github.com/rust-lang/rust/pull/132833
-                        if context.request_store.can_cancel(request_id) {
-                            ConfirmModal::new(
-                                "Cancel request?".into(),
-                                move |response| {
-                                    if response {
-                                        ViewContext::send_message(
-                                            Message::HttpCancel(request_id),
-                                        );
-                                    }
-                                },
-                            )
-                            .open();
-                        }
+                    if let Some(request_id) = self.selected_request_id.0
+                        && context.request_store.can_cancel(request_id)
+                    {
+                        ConfirmModal::new(
+                            "Cancel request?".into(),
+                            move |response| {
+                                if response {
+                                    ViewContext::send_message(
+                                        Message::HttpCancel(request_id),
+                                    );
+                                }
+                            },
+                        )
+                        .open();
                     }
                 }
                 Action::Quit => ViewContext::send_message(Message::Quit),
@@ -202,12 +203,6 @@ impl EventHandler for Root {
                     None
                 }
 
-                Event::Notify(notification) => {
-                    self.notification_text =
-                        Some(NotificationText::new(notification)).into();
-                    None
-                }
-
                 // Any other unhandled input event should *not* log an error,
                 // because it is probably just unmapped input, and not a bug
                 Event::Input { .. } => None,
@@ -222,6 +217,7 @@ impl EventHandler for Root {
         vec![
             self.modal_queue.to_child_mut(),
             self.primary_view.to_child_mut(),
+            self.footer.to_child_mut(),
         ]
     }
 }
@@ -251,15 +247,7 @@ impl<R: Deref<Target = RequestStore>> Draw<RootProps<R>> for Root {
         );
 
         // Footer
-        let footer = HelpFooter.generate();
-        let [notification_area, help_area] = Layout::horizontal([
-            Constraint::Min(10),
-            Constraint::Length(footer.width() as u16),
-        ])
-        .areas(footer_area);
-        self.notification_text
-            .draw_opt(frame, (), notification_area, false);
-        frame.render_widget(footer, help_area);
+        self.footer.draw(frame, (), footer_area, false);
 
         // Render modals last so they go on top
         self.modal_queue.draw(frame, (), frame.area(), true);
@@ -305,7 +293,6 @@ mod tests {
             test_util::TestComponent, util::persistence::DatabasePersistedStore,
         },
     };
-    use crossterm::event::KeyCode;
     use persisted::PersistedStore;
     use rstest::rstest;
     use slumber_core::{
@@ -314,6 +301,7 @@ mod tests {
         test_util::by_id,
     };
     use slumber_util::Factory;
+    use terminput::KeyCode;
 
     /// Test that, on first render, the view loads the most recent historical
     /// request for the first recipe+profile
@@ -580,17 +568,9 @@ mod tests {
         // Select "Delete Requests" but decline the confirmation
         component
             .int_props(props_factory)
-            .open_actions()
-            .send_keys([
-                // "Delete Request" action
-                KeyCode::Down,
-                KeyCode::Down,
-                KeyCode::Down,
-                KeyCode::Enter,
-                // Decline
-                KeyCode::Left,
-                KeyCode::Enter,
-            ])
+            .action("Delete Request")
+            // Decline
+            .send_keys([KeyCode::Left, KeyCode::Enter])
             .assert_empty();
 
         // Same request is still selected
@@ -603,16 +583,9 @@ mod tests {
         // for All Profiles
         component
             .int_props(props_factory)
-            .open_actions()
-            .send_keys([
-                // "Delete Request" action
-                KeyCode::Down,
-                KeyCode::Down,
-                KeyCode::Down,
-                KeyCode::Enter,
-                // Confirm
-                KeyCode::Enter,
-            ])
+            .action("Delete Request")
+            // Confirm
+            .send_keys([KeyCode::Enter])
             .assert_empty();
 
         assert_eq!(component.data().selected_request_id(), None);
@@ -657,15 +630,9 @@ mod tests {
         // Select "Delete Request" but decline the confirmation
         component
             .int_props(props_factory)
-            .open_actions()
-            .send_keys([
-                // "Delete Request" action
-                KeyCode::Up,
-                KeyCode::Enter,
-                // Decline
-                KeyCode::Left,
-                KeyCode::Enter,
-            ])
+            .action("Delete Request")
+            // Decline
+            .send_keys([KeyCode::Left, KeyCode::Enter])
             .assert_empty();
 
         // Same request is still selected
@@ -677,14 +644,9 @@ mod tests {
         component
             .int_props(props_factory)
             .send_key(KeyCode::Char('r'))
-            .open_actions()
-            .send_keys([
-                // "Delete Request" action
-                KeyCode::Up,
-                KeyCode::Enter,
-                // Confirm
-                KeyCode::Enter,
-            ])
+            .action("Delete Request")
+            // Confirm
+            .send_keys([KeyCode::Enter])
             .assert_empty();
 
         // New exchange is gone
