@@ -238,7 +238,7 @@ pub enum Event {
     /// Input from the user, which may or may not correspond to a bound action.
     /// Most components just care about the action, but some require raw input
     Input {
-        event: crossterm::event::Event,
+        event: terminput::Event,
         action: Option<Action>,
     },
 
@@ -262,6 +262,8 @@ pub enum Event {
 
     /// Tell the user something informational
     Notify(Notification),
+    /// Clear the current notification if it matches the given ID
+    NotifyClear(Uuid),
 
     /// A localized event emitted by a particular [Emitter] implementation.
     /// The event type here does not need to be unique because the emitter ID
